from src.dbdisk.db_disk_request import DbDiskRequest
from src.dbdisk.types import DbType, DiskFileType

MAX_ROWS = 1000000


class DbDiskCacheBuilder:
    def __init__(self):
        self.db_type = DbType.NONE
        self.disk_file_type = DiskFileType.CSV
        self.cache_path = None
        self.cache_name = None
        self.connection_string = None
        self.db_type = None
        self.can_zip = False
        self.rows_per_file = MAX_ROWS

    @classmethod
    def create(cls, setup):
        builder = cls()
        setup(builder)
        return builder

    def set_db_type(self, db_type: DbType):
        self.db_type = db_type
        return self

    def set_disk_file_type(self, disk_file_type: DiskFileType):
        self.disk_file_type = disk_file_type
        return self

    def set_cache_path(self, path):
        self.cache_path = path
        return self

    def set_cache_name(self, name):
        self.cache_name = name
        return self

    def set_connection_string(self, connection_string):
        self.connection_string = connection_string
        return self

    def set_can_zip(self, can_zip):
        self.can_zip = can_zip
        return self

    def set_rows_per_file(self, rows_per_file):
        self.rows_per_file = rows_per_file
        return self

    def execute(self, query):
        print(f"Executing query: {query}")
        print(f"Using cache path: {self.cache_path}")
        print(f"Cache name: {self.cache_name}")
        print(f"Connection string: {self.connection_string}")
        print(f"Database type: {self.db_type}")
        return DbDiskRequest(self.connection_string, self.db_type, self.cache_path, self.cache_name, self.disk_file_type,
                             self.can_zip, self.rows_per_file).execute(query)
        # Add actual database execution logic here



from urllib.parse import urljoin
import requests
import configparser
from retry import retry


class HttpRequest:
    def __init__(self):
        self.base_url = ""
        self.headers = {'User-Agent': 'MyApp/1.0'}

    def __str__(self):
        return f'{self.method} {self.base_url}'

    @property
    def tries(self):
        return self.tries

    # Decorate your function with the @retry decorator
    @retry(delay=1, backoff=2, max_delay=4, tries=3)
    def request_get(self, url):
        full_url = urljoin(self.base_url, url)
        return requests.get(full_url, headers=self.headers or None)

    def set_base_url(self, url):
        self.base_url = url
        return self

    def set_headers(self, headers):
        self.headers = headers
        return self

    def build(self):
        return self


from abc import ABC, abstractmethod


class DbDiskCache:
    def __init__(self, cache_dir, cache_name, can_zip=False, rows_per_file=1000000):
        self.cache_dir = cache_dir
        self.cache_name = cache_name
        self.rows_per_file = rows_per_file
        self.can_zip = can_zip

    @abstractmethod
    def save(self, header, data):
        pass



from abc import abstractmethod

import openai


class OpenAIBotBase:
    def __init__(self, api_key):
        self.api_key = api_key
        self.client = None

    def get_response(self, response):
        return response.choices[0].message.content.strip()

    @staticmethod
    def create_message(prompt):
        return [{"role": "user", "content": prompt},
                {"role": "system",
                 "content": "You are a helpful assistant that provides concise and accurate information."}]

    @staticmethod
    def handle_error(self, error):
        error_messages = {
            openai.RateLimitError: "Rate limit exceeded. Please try again later.",
            openai.AuthenticationError: "Authentication failed. Please check your API key.",
            openai.OpenAIError: "An error occurred."
        }

        error_type = type(error)
        message = error_messages.get(error_type, "An unknown error occurred.")
        raise Exception(message, error)



from src.dbdisk.database.db_service import DbService
from src.dbdisk.database.db_service_factory import DbServiceFactory
from src.dbdisk.db_disk_factory import DbDiskFactory


class DbDiskRequest:
    def __init__(self, connection_string, db_type, cache_dir, cache_name, disk_file_type, can_zip, rows_per_file):
        self.connection_string = connection_string
        self.db_type = db_type
        self.cache_dir = cache_dir
        self.cache_name = cache_name
        self.disk_file_type = disk_file_type
        self.can_zip = can_zip
        self.rows_per_file = rows_per_file

    def execute(self, query):
        db_service = DbServiceFactory.create_db_service(self.db_type, self.connection_string)
        header, data = db_service.execute(query)
        print(data)
        return DbDiskFactory.create_db_disk(self.disk_file_type, self.cache_dir, self.cache_name, self.can_zip,
                                            self.rows_per_file).save(header, data)



import shutil
from itertools import islice


def split_into_subsets(data, n):
    """
    Split the data into n subsets.
    """
    subsets = []
    iterable = iter(data)
    subset = list(islice(iterable, n))
    while subset:
        subsets.append(subset)
        subset = list(islice(iterable, n))
    return subsets

def zip_directory(directory, zip_name, delete_after_zip=True):
    """
    Zip the directory.
    """
    shutil.make_archive(zip_name, 'zip', directory)
    if delete_after_zip:
        shutil.rmtree(directory)


from openai import OpenAI
from src.openai.openai_bot_base import OpenAIBotBase


class OpenAIBot(OpenAIBotBase):
    def __init__(self, api_key):
        super().__init__(api_key)
        self.client = OpenAI(api_key=api_key)

    def get_response(self, prompt: str, model: str = "gpt-3.5-turbo"):
        try:
            response = self.client.chat.completions.create(
                model=model,
                messages=self.create_message(prompt))
            return super().get_response(response)
        except Exception as e:
            self.handle_error(e)
            return None





from src.dbdisk.database.db_service import DbService


class DbPgService(DbService):
    def __init__(self, connection_string):
        super().__init__(connection_string)

    def connect(self):
        import psycopg2
        return psycopg2.connect(self.connection_string)




from openai import AsyncOpenAI
from typing import Callable, Any
from src.openai.openai_bot_base import OpenAIBotBase


class AsyncOpenAIBot(OpenAIBotBase):

    def __init__(self, api_key):
        super().__init__(api_key)
        self.client = AsyncOpenAI(api_key=api_key)

    async def get_response_async(self, prompt: str, callback: Callable[[Any], None], model: str = "gpt-3.5-turbo"):
        try:
            response = await self.client.chat.completions.create(
                model=model,
                messages=self.create_message(prompt))
            callback(super().get_response(response))
        except Exception as e:
            self.handle_error(e)




import os
import xml.etree.ElementTree as ET


class XmlCreator:
    @staticmethod
    def create_xml_file(file_dir, file_name, **kwargs):
        root = ET.Element("root")
        # Create sub elements with the provided tags and values
        for tag, value in kwargs.items():
            element = ET.SubElement(root, tag)
            element.text = str(value)

        # Create an ElementTree object with the root element
        tree = ET.ElementTree(root)

        os.makedirs(file_dir, exist_ok=True)
        file_path = os.path.join(file_dir, file_name)

        # Write the XML content to the file
        with open(file_path, 'wb') as file:
            tree.write(file)




from src.dbdisk.database.db_oracle_service import DbOracleService
from src.dbdisk.database.db_pg_service import DbPgService
from src.dbdisk.database.db_service import DbService
from src.dbdisk.database.db_sybase_service import DbSybaseService
from src.dbdisk.types import DbType


class DbServiceFactory:
    @staticmethod
    def create_db_service(db_type: DbType, connection_string: str) -> DbService:
        if db_type == DbType.POSTGRESQL:
            return DbPgService(connection_string)
        elif db_type == DbType.SYBASE:
            return DbSybaseService(connection_string)
        elif db_type == DbType.ORACLE:
            return DbOracleService(connection_string)
        else:
            raise ValueError(f"Unsupported database type: {db_type}")


from setuptools import setup, find_packages

setup(
    name='omnijp',
    version='2.10.0',
    packages=find_packages(),
    install_requires=[
        'retry>=0.9.2',
        'requests>=2.31.0',
        'pyodbc>=5.1.0',
        'psycopg2>=2.9.9',
        'openai~=1.30.5',
        'cx_Oracle~=8.3.0'
    ],
    entry_points={
        'console_scripts': [
            # Add any command-line scripts here
        ],
    },
    # Additional metadata
    author='Jessish Pothancheri',
    author_email='jessish.pothancheri@gmail.com',
    description='OmniJP is a Python library that provides tools for common tasks in software development',
    long_description=open('README.md').read(),
    long_description_content_type='text/markdown',
    url='https://github.com/jpothanc/omnijp',
    license='MIT',
    classifiers=[
        'Development Status :: 5 - Production/Stable',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Programming Language :: Python :: 3.8',],
    python_requires='>=3.8',
)



import os
import csv
from src.dbdisk.caches.db_disk_cache import DbDiskCache
from src.common.helper import split_into_subsets, zip_directory


class DbDiskCacheCsv(DbDiskCache):
    def save(self, header, data):
        try:
            local_cache_dir = self.get_local_cache_path()
            os.makedirs(local_cache_dir, exist_ok=True)
            subsets = split_into_subsets(data, self.rows_per_file)
            count = 1
            total = 0
            for subset in subsets:
                file_path = os.path.join(local_cache_dir, f"{self.cache_name}_{count}.csv")
                self.save_file(header, subset, file_path)
                count += 1
                total += len(subset)
            print(f"Total records saved: {total}")
            if self.can_zip:
                zip_directory(local_cache_dir, local_cache_dir)
        except Exception as e:
            print(f"Error saving cache: {e}")
            return False
        print(f"Cache saved to {file_path}")
        return True

    def get_local_cache_path(self):
        return os.path.join(self.cache_dir, self.cache_name) if self.can_zip else self.cache_dir

    @staticmethod
    def save_file(header, data, file_path):
        try:
            with open(file_path, 'w', newline='') as csv_file:
                csv_writer = csv.writer(csv_file)
                csv_writer.writerow(header)
                for row in data:
                    csv_writer.writerow(row)
        except Exception as e:
            print(f"Error saving cache: {e}")
            return False
        print(f"Cache saved to {file_path}")
        return True



from src.dbdisk.caches.db_disk_cache_csv import DbDiskCacheCsv
from src.dbdisk.types import DiskFileType


class DbDiskFactory:
    @staticmethod
    def create_db_disk(disk_file_type, cache_dir, cache_name, can_zip=False, rows_per_file=1000000):
        if disk_file_type == DiskFileType.CSV:
            return DbDiskCacheCsv(cache_dir, cache_name, can_zip, rows_per_file)
        elif disk_file_type == DiskFileType.JSON:
            raise NotImplementedError
        elif disk_file_type == DiskFileType.XML:
            raise NotImplementedError
        else:
            return None


class DbOracleService:
    def __init__(self, connection_string: str):
        self.connection_string = connection_string

    def connect(self, query: str):
        import cx_Oracle
        return cx_Oracle.connect(self.connection_string)



from abc import abstractmethod

import psycopg2
import pymssql


class DbService:

    def __init__(self, connection_string):
        self.connection_string = connection_string

    def execute(self, query):
        connection = None
        cursor = None
        try:
            print("Executing query: ", query)
            print("Connecting to database...")
            connection = self.connect()
            print("Connected to database")
            cursor = connection.cursor()
            print("Executing query...")
            cursor.execute(query)
            header = [desc[0] for desc in cursor.description]
            result = cursor.fetchall()
            print("Query executed")
        except Exception as e:
            self.handle_error(e)
            result = None
            header = None
        finally:
            if cursor:
                cursor.close()
            if connection:
                connection.close()
        return header, result

    @abstractmethod
    def connect(self):
        pass

    @staticmethod
    def handle_error(error):
        error_messages = {
            psycopg2.Error: "Error connecting to PostgresSQL:",
            pymssql.Error: "Error connecting to Sybase:"
        }
        error_type = type(error)
        message = error_messages.get(error_type, "Unknown error")
        raise Exception(message, error)
